@@ -1,9 +1,10 @@
-from csc.divisi.tensor import DictTensor
+from csc.divisi.tensor import DictTensor, Tensor
 from csc.divisi.ordered_set import OrderedSet
 from csc.divisi.labeled_view import LabeledView
+from csc.divisi.normalized_view import MeanSubtractedView
 from itertools import chain, izip
 import logging
-from math import sqrt
+from math import sqrt, ceil
 
 def partial_list_repr(lst, max_len):
     if len(lst) <= max_len:
@@ -93,6 +94,8 @@ class Blend(LabeledView):
         You can pass a ``dict`` or sequence of ``(label, tensor)``
         pairs; the tensors will be labeled according to the keys.
         '''
+        if isinstance(tensors, Tensor):
+            raise TypeError('Give Blend a _list_ (or dict or whatever) of tensors.')
         if hasattr(tensors, 'items'):
             # Extract the items, if we have some.
             tensors = tensors.items()
@@ -102,6 +105,11 @@ class Blend(LabeledView):
             names, tensors = zip(*tensors)
         else:
             names = map(repr, tensors)
+
+        for tensor in tensors:
+            if tensor.stack_contains(MeanSubtractedView):
+                raise TypeError("You can't blend MeanSubtractedViews. Try mean-subtracting the resulting blend.")
+
         self._tensors = tuple(tensors)
         self.names = tuple(names)
         self.logger.info('tensors: %s', ', '.join(self.names))
@@ -344,8 +352,8 @@ class Blend(LabeledView):
 
         ``width`` and/or ``height``: the target width and height of
           the image. If it doesn't fit evenly, the image may be
-          slightly bigger than you specify. Defaults to square if one
-          or the other is unspecified.
+          slightly bigger than you specify. Defaults to square pixels
+          if one or the other is unspecified.
 
         Or if you give no parameters, the width defaults to 1000 pixels.
 
@@ -360,18 +368,19 @@ class Blend(LabeledView):
             # Compute the dimensions that are specified.
             pixel_width = pixel_height = None
             if width is not None:
-                pixel_width = self.shape[1] // width
+                pixel_width = int(ceil(float(self.shape[1]) / width))
             if height is not None:
-                pixel_height = self.shape[0] // height
+                pixel_height = int(ceil(float(self.shape[0]) / height))
 
             # Fill in, defaulting to square.
-            pixel_size = (pixel_width  if pixel_width  is not None else pixel_height,
-                          pixel_height if pixel_height is not None else pixel_width)
+            pixel_size = (pixel_height if pixel_height is not None else pixel_width,
+                          pixel_width  if pixel_width  is not None else pixel_height)
 
         else:
             if width is not None or height is not None:
                 raise TypeError("Can't specify both pixel_size and width/height.")
 
+        self.logger.debug('Making coverage image with pixel_size=%r', pixel_size)
         # Generate the raw coverage data.
         fill, magnitude, src = self.coverage(pixel_size)
 
