@@ -204,7 +204,7 @@ class IdentitySet(object):
     def __getstate__(self): return dict(len=self.len)
     def __setstate__(self, state): self.len = state['len']
 
-def indexable_set(x, dim):
+def indexable_set(x, dim=None):
     if x is None:
         return IdentitySet(dim)
     if getattr(x, 'index_is_efficient', False):
