@@ -115,12 +115,15 @@ class SVD2DResults(object):
         self.u = u
         self.v = v
         self.svals = svals
-        self.reconstructed = Reconstructed2DTensor(self)
         self.clear_cache()
 
     def __repr__(self):
         return u'<SVD2DResults: %d by %d, rank=%d>' % (self.u.shape[0], self.v.shape[0], len(self.svals))
 
+    @property
+    def reconstructed(self):
+        return Reconstructed2DTensor(self)
+
     def clear_cache(self):
         '''
         Delete all cached tensors from this object.
@@ -144,9 +147,12 @@ class SVD2DResults(object):
         matrix by its corresponding sigma value.
         '''
         if self._weighted_u is None:
-            self._weighted_u = self.u*self.core
+            self._weighted_u = self.get_weighted_u()
         return self._weighted_u
 
+    def get_weighted_u(self):
+        return DenseTensor(self.u.unwrap() * self.svals.unwrap())
+
     @property
     def weighted_v(self):
         '''
@@ -154,16 +160,19 @@ class SVD2DResults(object):
         matrix by its corresponding sigma value.
         '''
         if self._weighted_v is None:
-            self._weighted_v = self.v*self.core
+            self._weighted_v = self.get_weighted_v()
         return self._weighted_v
 
+    def get_weighted_v(self):
+        return DenseTensor(self.v.unwrap() * self.svals.unwrap())
+
     def u_dotproducts_with(self, vec):
         '''
         Return the dot product of *vec* with every weighted *u*
         vector.
         '''
         return self.weighted_u * vec
-
+    
     def u_angles_to(self, vec):
         '''
         Return the cosine of the angle between *vec* and every weighted
@@ -172,10 +181,10 @@ class SVD2DResults(object):
         '''
         u = self.weighted_u
         angles = self.u_dotproducts_with(vec)
-        vec_magnitude = sqrt(vec*vec)
+        vec_magnitude = vec.norm()
         # Normalize distances to get the cos(angles)
         for key, value in angles.iteritems():
-            u_vector_magnitude = sqrt(u[key[0], :]*u[key[0], :])
+            u_vector_magnitude = self.weighted_u_vec(key[0]).norm()
             angles[key] = float(value)/(u_vector_magnitude*vec_magnitude)
 
         return angles
@@ -325,7 +334,7 @@ class SVD2DResults(object):
         *u* and *v* matrices with the largest components on the axis.
         '''
         if k is None: k = len(self.svals)
-        else: k = min(k, len(self.svals))
+        else: k = min(int(k), len(self.svals))
         for axis in range(k):
             self.summarize_axis(axis, k, u_only, screen_width, output)
 
@@ -336,11 +345,10 @@ class SVD2DResults(object):
         return numpy.array([float(self.svals[i]) for i in range(len(self.svals))])
 
     def export_svdview(self, outfn, packed=True, **kw):
-        ''' Output a tab-separated values file suitable for use with
-        svdview. The data is saved to the file named _outfn_. See the
-        documentation in :mod:`export_svdview
-        <csc.divisi.export_svdview>` for more information on the
-        ``denormalize`` and ``packed`` options.
+        ''' Output a data file suitable for use with svdview. The data
+        is saved to the file named _outfn_. See the documentation in
+        :mod:`export_svdview <csc.divisi.export_svdview>` for more
+        information on the ``denormalize`` and ``packed`` options.
         '''
         import export_svdview
         if packed:
@@ -348,45 +356,50 @@ class SVD2DResults(object):
         else:
             export_svdview.write_tsv(self.u, outfn, **kw)
 
+    def slice(self, k=None, rows=None, cols=None):
+        '''
+        Slice out a part of the SVD results, fewer svals, rows, and/or columns.
 
-    # Deprecated stuff below here.
-    @property
-    def ut(self):
-        warn(DeprecationWarning("ut and vt are incorrect names, use u and v."))
-        return self.u
+        k : int - how many svals to keep. None -> keep all.
+        rows : [int] - a list of rows to keep. None -> keep all.
+        cols : [int] - a list of cols to keep. None -> keep all.
+        '''
+        u, v, svals = self.u.unwrap(), self.v.unwrap(), self.svals.unwrap()
+        if svals is not None:
+            svals = svals[:k]
+            u = u[:,:k]
+            v = v[:,:k]
+        if rows is not None:
+            u = u[rows,:]
+        if cols is not None:
+            v = v[cols,:]
+        return SVD2DResults(DenseTensor(u), DenseTensor(v), DenseTensor(svals))
 
-    @property
-    def vt(self):
-        warn(DeprecationWarning("ut and vt are incorrect names, use u and v."))
-        return self.v
+    def denormalized(self, norms):
+        row_norms, col_norms = norms
+        u, v, svals = self.u.unwrap(), self.v.unwrap(), self.svals.unwrap()
 
-    @property
-    def weighted_ut(self):
-        warn(DeprecationWarning("ut and vt are incorrect names, use u and v."))
-        return self.weighted_u()
+        import numpy as np
+        if row_norms is not None:
+            u = self.u.unwrap() * row_norms[:, np.newaxis]
+        if col_norms is not None:
+            v = self.v.unwrap() * col_norms[:, np.newaxis]
+        return SVD2DResults(DenseTensor(u), DenseTensor(v), DenseTensor(svals))
+        
 
-    @property
-    def weighted_vt(self):
-        warn(DeprecationWarning("ut and vt are incorrect names, use u and v."))
-        return self.weighted_v()
+    # TODO: u_distances_to and v_distances_to had these notes:
 
-    def u_distances_to(self, vec):
-        # Once the old use of this method is removed, we can replace
-        # it with something that actually behaves according to its
-        # name, i.e., returns _||vec - ((vec * u) * u^hat)||_, that is
-        # it returns the magnitude of the component of _vec_
-        # orthogonal to _u_.
-        warn(DeprecationWarning('u_distances_to is named incorrectly; use u_dotproducts_with'))
-        return self.u_dotproducts_with(vec)
+    # Once the old use of this method is removed, we can replace
+    # it with something that actually behaves according to its
+    # name, i.e., returns _||vec - ((vec * u) * u^hat)||_, that is
+    # it returns the magnitude of the component of _vec_
+    # orthogonal to _u_.
 
-    def v_distances_to(self, vec):
-        # Once the old use of this method is removed, we can replace
-        # it with something that actually behaves according to its
-        # name, i.e., returns _||vec - ((vec * u) * u^hat)||_, that is
-        # it returns the magnitude of the component of _vec_
-        # orthogonal to _u_.
-        warn(DeprecationWarning('v_distances_to is named incorrectly; use v_dotproducts_with'))
-        return self.v_dotproducts_with(vec)
+    # Once the old use of this method is removed, we can replace
+    # it with something that actually behaves according to its
+    # name, i.e., returns _||vec - ((vec * u) * u^hat)||_, that is
+    # it returns the magnitude of the component of _vec_
+    # orthogonal to _u_.
 
 
 class Reconstructed2DTensor(Tensor):
@@ -601,3 +614,4 @@ def incremental_svd(tensor, k=50, niter=100, lrate=.1, logging_interval=None):
 
     u, v, sigma = tensor._isvd(k=k, niter=niter, lrate=lrate)
     return SVD2DResults(u, v, sigma)
+
