@@ -15,7 +15,7 @@ store data sparsely in dictionaries; then, there are various kinds of
 import numpy, sys
 from csc.divisi.dict_mixin import MyDictMixin
 import copy, random, math
-from csc.divisi.exceptions import DimensionMismatch
+from csc.divisi.exceptions import DimensionMismatch, InvalidLayeringException
 from csc.divisi._svdlib import svd, dictSvd, isvd, dictIsvd
 from warnings import warn
 import logging
@@ -179,6 +179,20 @@ class Tensor(MyDictMixin):
         """
         return cls(self, *a, **kw)
 
+    def stack_contains(self, cls):
+        '''
+        Check if the tensor stack below here includes an instance of ``cls``.
+        
+        You may pass a tuple of classes as well, which will check if
+        the stack contains _any_ of those classes.
+        '''
+        if isinstance(self, cls):
+            return True
+        if hasattr(self, 'tensor'):
+            return self.tensor.stack_contains(cls)
+        else:
+            return False
+
     def unfolded(self, mode):
         """
         Get an :class:`UnfoldedView <divisi.unfolded_view.UnfoldedView>` of
@@ -228,9 +242,9 @@ class Tensor(MyDictMixin):
         raise TypeError('Unknown normalization parameter %r' % mode)
             
 
-    def zero_mean_normalized(self, mode=0):
-        from csc.divisi.normalized_view import ZeroMeanView
-        return self.add_layer(ZeroMeanView, mode)
+    def mean_subtracted(self, mode=0):
+        from csc.divisi.normalized_view import MeanSubtractedView
+        return self.add_layer(MeanSubtractedView, mode)
 
     # Math
     def __add__(self, other):
@@ -559,9 +573,32 @@ class Tensor(MyDictMixin):
         else:
             return res
 
+    def means(self):
+        '''
+        Computes the by-slice means for each dimension.
+
+        Returns numpy arrays, not DenseTensors.
+        '''
+        import numpy as np
+        accumulator_for_mode = [numpy.zeros(n) for n in self.shape]
+        for k, v in self.iteritems():
+            for accumulator, key in izip(accumulator_for_mode, k):
+                accumulator[key] += v
+        
+        # Divide by the number of items that could have gone into each
+        # accumulator slot.
+        total_possible_items = np.product(self.shape)
+        for mode, accumulator in enumerate(accumulator_for_mode):
+            possible_items = total_possible_items / self.shape[mode]
+            accumulator /= possible_items
+        return accumulator_for_mode
+            
+
     def label_histograms(self):
         '''
-        Counts the number and magnitude of items in each row / column
+        Counts the number and magnitude of items in each row / column.
+
+        Returns (counts, magnitudes).
         '''
         import numpy
         counts = [numpy.zeros(n, dtype=numpy.uint16) for n in self.shape]
@@ -576,9 +613,16 @@ class Tensor(MyDictMixin):
         return (map(DenseTensor, counts),
                 map(DenseTensor, magnitudes))
 
-
-class InvalidLayeringException(StandardError):
-    pass
+    # Sparse shape handling (TODO: consistent semantics)
+    def ensure_index_is_valid(self, idx):
+        '''
+        Ensure that an index is valid, possibly updating the shape to reflect that.
+        '''
+        # Subclasses with adjustable shape should override this.
+        shape = self.shape
+        for mode in xrange(self.ndim):
+            if idx[mode] >= shape[mode]:
+                raise ValueError('Index (%d) is out of range (%d) for dimension %d' % (idx[mode], shape[mode], mode))
 
 forward_to_tensor = set([
         'shape', 'ndim', 'has_key', 'default_value', 'density'])
@@ -1133,6 +1177,14 @@ class DictTensor(Tensor):
             res[key] = -value
         return res
 
+    def ensure_index_is_valid(self, indices):
+        # A bit of code in __setitem__ duplicates this for speed.
+        s = self._shape
+        for mode, idx in enumerate(indices):
+            assert isinstance(idx, (int, long))
+            if idx >= s[mode]:
+                s[mode] = idx + 1
+
     def _svd(self, k=50, *a, **kw):
         return dictSvd(self, k=k, *a, **kw)
 
