@@ -55,6 +55,8 @@ SMat svdNewSMat(int rows, int cols, int vals) {
   if (!S->rowind) {svdFreeSMat(S); return NULL;}
   S->value  = svd_doubleArray(vals, FALSE, "svdNewSMat: value");
   if (!S->value)  {svdFreeSMat(S); return NULL;}
+  S->offset_for_row = NULL;
+  S->offset_for_col = NULL;
   return S;
 }
 
@@ -63,6 +65,8 @@ void svdFreeSMat(SMat S) {
   SAFE_FREE(S->pointr);
   SAFE_FREE(S->rowind);
   SAFE_FREE(S->value);
+  SAFE_FREE(S->offset_for_row);
+  SAFE_FREE(S->offset_for_col);
   free(S);
 }
 
@@ -158,10 +162,22 @@ SMat svdTransposeS(SMat S) {
       N->value[j] = S->value[i];
     }
   }
+  /* Transpose the row and column offsets also. */
+  if (S->offset_for_col)
+    N->offset_for_row = copyVector(S->offset_for_col, S->cols, "svdTransposeS: offset_for_row");
+  if (S->offset_for_row)
+    N->offset_for_col = copyVector(S->offset_for_row, S->rows, "svdTransposeS: offset_for_row");
+
   return N;
 }
 
-
+double *copyVector(double* vec, int n, const char* name) {
+  int i;
+  double* result = svd_doubleArray(n, FALSE, name);
+  for (i = 0; i < n; i++)
+    result[i] = vec[i];
+  return result;
+}
 
 /* this stuff was added by Rob */
 void freeVector(double *v) {
