@@ -11,6 +11,8 @@ def get_shape_for_dict(expected):
 
 def nones_removed(d):
     return dict((k, v) for k, v in d.iteritems() if v is not None)
+def zeros_removed(d):
+    return dict((k, v) for k, v in d.iteritems() if v)
 
 def dict_fill_in_missing_dims(d):
     items = d.items()
