@@ -24,6 +24,11 @@ def test_bare_blend():
     b = Blend([t1], weights=[1])
     assertSetEquals(set(t1.label_list(0)), set(b.label_list(0)))
 
+@raises(TypeError)
+def test_nonlist():
+    Blend(t1, weights=[1])
+    
+
 def test_manual_weights():
     '''
     Specifying weights manually causes the result to be the weighted sum.
@@ -135,6 +140,9 @@ class MockTensor(object):
         import numpy
         from csc.divisi.tensor import DenseTensor
         return MockSVDResults(None, DenseTensor(numpy.array(self.svals[:k])), None)
+    def stack_contains(self, cls):
+        from csc.divisi.labeled_view import make_sparse_labeled_tensor
+        return make_sparse_labeled_tensor(2).stack_contains(cls)
 
 class MockSVDResults(object):
     def __init__(self, u, svals, v):
